/**
 ** mocapMainTaskCodels.cc
 **
 ** Codels called by execution task mocapMainTask
 **
 ** Author: 
 ** Date: 
 **
 **/

#include <portLib.h>
#include "evart-client.h"
#include "mocapConst.h"
#include "mocapStruct.h"
#include "server/mocapHeader.h"

// error Codes
// status == 0 OK
// status == 1 Error initiating evas_body_markers
// status == 2 body_markers.fDelay > 0.15
// status == 3 Invalid iFrame
// status == 4 Evas_Empty
int iframe = -1;
int getMeasurement(MOCAP_MARKERS_OBJECT *mocapObject, double time,int objectNumber){


  int numMarker = 0;
  evas_msg_t msgStruct;
  evas_msg_t *msg = NULL;


  if(evas_body_markers(objectNumber, EVAS_ON) != 0){
    printf("No connection to %s:%d", EVAS_STREAM_HOST, EVAS_STREAM_PORT);
    printf("Error initiating evas_body_markers, trying again\n");
    return 1;
  }

  msg = ( evas_msg_t *) evas_recv(&msgStruct,time);


  while((msg == NULL) || !(msg->type == EVAS_BODY_MARKERS)){
    printf("Wrong Message, try again, and the type\n");
    msg = ( evas_msg_t *) evas_recv(&msgStruct,time);
  }

  evas_body_markers(objectNumber, EVAS_OFF);

  if ((iframe < msg->body_markers.iFrame) || ((iframe > 1000000) && (msg->body_markers.iFrame > 100000))){

    iframe = msg->body_markers.iFrame;

    if(msg->body_markers.fDelay > 0.15){
      printf("fDelay > 0.15\n");
      return 2;
    }
    //if (msg->type == EVAS_BODY_MARKERS) {
    for (numMarker = 0; numMarker < msg->body_markers.nmarkers; numMarker++) {


      if(msg->body_markers.markers[numMarker][0] == EVAS_EMPTY){
        ///	printf("EVAS_EMPTY\n");
        return 4;
      }

      mocapObject->x[numMarker] = msg->body_markers.markers[numMarker][0]/1000;
      mocapObject->y[numMarker] = msg->body_markers.markers[numMarker][1]/1000;
      mocapObject->z[numMarker] = msg->body_markers.markers[numMarker][2]/1000;
    }
  }else{
    return 3;
  }
  return 0;
}


/*------------------------------------------------------------------------
 * Init
 *
 * Description: 
 *
 * Reports:      OK
 */

/* mocapInitMain  -  codel EXEC of Init
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mocapInitMain(int *report)
{
  /* initialize internal data by sending a hello msg */
  if(evas_text("hello") != 0) {
    printf("Not hello ServeR turn the motion capture system and run the cortex-server\n");
    *report = S_mocap_MOTION_SYSTEM_NOT_FOUND;
    return ETHER;
  }

  /* read body list name */
  const evas_body_list_t *bl = evas_body_list();
  const evas_body_markers_list_t* markerList;
  if (!bl) {
    printf("Not Lecture on the server not possible\n");
    *report = S_mocap_MOTION_SYSTEM_NOT_FOUND;
    return ETHER;
  }else{
    printf("EVART: number of bodies %d\n",bl->nbodies);
    SDI_F->objects.nbodies = bl->nbodies;
    for (int i = 0; i < bl->nbodies; i++){
      printf("Tracked Body: %s, Body number = %d\n",bl->bodies[i],i);
      strcpy(SDI_F->objects.objects[i].name,bl->bodies[i]);
      
      markerList =  evas_body_markers_list(i);
      SDI_F->objects.objects[i].nmarkers = markerList->nmarkers;

      printf("Number of Markers: %d\n",markerList->nmarkers);

      for(int j = 0; j < markerList->nmarkers; j++){
        printf("Marker Name: %s\n",markerList->markers[j]);
      }
    }
  }
  return ETHER;
}

/*------------------------------------------------------------------------
 * ReadMarkers
 *
 * Description: 
 *
 * Reports:      OK
 *              S_mocap_MODULE_NOT_INITIALIZED
 *              S_mocap_MOTION_SYSTEM_NOT_FOUND
 */


/* mocapReadMarkersMain  -  codel EXEC of ReadMarkers
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mocapReadMarkersMain(int *report)
{

  int status;

  for(int b = 0;b<SDI_F->objects.nbodies;b++){    
    status = getMeasurement(&SDI_F->objects.objects[b],1,b);

    if ((status == 0)){
      SDI_F->objects.objects[b].status = status;
    }else{
      //printf("badLectures on body == %d, poster not actualized\n",b);
      SDI_F->objects.objects[b].status = status;
      continue;
    }
  }
  return EXEC;
}

/* mocapReadMarkersEnd  -  codel END of ReadMarkers
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mocapReadMarkersEnd(int *report)
{
  for(int b = 0;b<SDI_F->objects.nbodies;b++){
    evas_body_markers(b, EVAS_OFF);
  }
  return ETHER;
}

/* mocapReadMarkersInter  -  codel INTER of ReadMarkers
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mocapReadMarkersInter(int *report)
{
  for(int b = 0;b<SDI_F->objects.nbodies;b++){
    evas_body_markers(b, EVAS_OFF);
  }
  return ETHER;
}







/*------------------------------------------------------------------------
 * Get4MarkerPose
 *
 * Description: 
 *
 * Reports:      OK
 *              S_mocap_WRONG_OBJECT_ID
 */

/* mocapGet4MarkerPoseMain  -  codel EXEC of Get4MarkerPose
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mocapGet4MarkerPoseMain(int *objectId, int *report)
{
  int i=0;
  for(i=0; i<4;i++) {
    printf(" mrkrsBasePos[%d].x = %f\n", i, SDI_F->objects.objects[*objectId].x[i]);
    printf(" mrkrsBasePos[%d].y = %f\n", i, SDI_F->objects.objects[*objectId].y[i]);
    printf(" mrkrsBasePos[%d].z = %f\n", i, SDI_F->objects.objects[*objectId].z[i]);
  }
  return ETHER;
}
